@@ -32,12 +32,12 @@ import time
 
 import gobject
 import gtk
-import gtk.keysyms 
+from gtk import keysyms 
 
 from kiwi import ValueUnset
 from kiwi.interfaces import implementsIProxy, implementsIMandatoryProxy
 from kiwi.ui.widgets.proxy import WidgetMixin, WidgetMixinSupportValidation
-from kiwi.utils import gsignal
+from kiwi.utils import gsignal, gproperty
 
 (COL_COMBO_LABEL,
  COL_COMBO_DATA) = range(2)
@@ -209,6 +209,9 @@ class ComboProxyMixin(object):
             raise KeyError("No item correspond to data %r in the combo %s" 
                            % (data, self.name))
 
+    def get_model_strings(self):
+        return [row[COL_COMBO_LABEL] for row in self.get_model()]
+
     def get_model_items(self):
         if self.mode != COMBO_MODE_DATA:
             raise TypeError("get_model_items can only be used in data mode")
@@ -251,6 +254,7 @@ class ComboBox(gtk.ComboBox, ComboProxyMixin, WidgetMixin):
         renderer = gtk.CellRendererText()
         self.pack_start(renderer)
         self.add_attribute(renderer, 'text', COL_COMBO_LABEL)
+        self.show()
 
     def do_changed(self):
         self.emit('content-changed')
@@ -267,7 +271,7 @@ class ComboBox(gtk.ComboBox, ComboProxyMixin, WidgetMixin):
     def update(self, data):
         # We dont need validation because the user always
         # choose a valid value
-        
+
         WidgetMixin.update(self, data)
         
         if data is ValueUnset or data is None:
@@ -301,6 +305,8 @@ class ComboBoxEntry(gtk.ComboBoxEntry, ComboProxyMixin,
     # not the combo box itself.
     #gsignal('expose-event', 'override')
     
+    gproperty("list-writable", bool, False, 
+              "List Writable", gobject.PARAM_READWRITE)
     def __init__(self):
         gtk.ComboBoxEntry.__init__(self)
         WidgetMixinSupportValidation.__init__(self, widget=self.child)
@@ -318,6 +324,19 @@ class ComboBoxEntry(gtk.ComboBoxEntry, ComboProxyMixin,
         self.set_events(gtk.gdk.KEY_RELEASE_MASK)
         self.connect("key-release-event", self._on__key_release_event)
     
+        self._list_writable = True
+        self.show()
+    
+    def prop_get_list_writable(self):
+        return self._list_writable
+    
+    def prop_set_list_writable(self, writable):
+        if self.mode == COMBO_MODE_DATA:
+            return
+        
+        self.child.set_editable(writable)
+        self._list_writable = writable
+
     def _update_selection(self, text=None):
         if text is None:
             text = self.child.get_text()
@@ -329,20 +348,22 @@ class ComboBoxEntry(gtk.ComboBoxEntry, ComboProxyMixin,
         if not text.strip():
             return
 
-        # XXX: Fix this to work in string and data mode
-        return
-    
-        if text in self.get_model_items():
+        if text in self.get_model_strings():
             return
         
+        self.child.set_text('')
         self.append_item(text)
         self._update_selection(text)
-     
+        
     def _on__key_release_event(self, widget, event):
         """Checks for "Enter" key presses and add the entry text to 
         the combo list if the combo list is set as editable.
         """
-        if event.keyval in (gtk.keysyms.KP_Enter, gtk.keysyms.Return):
+        if not self._list_writable:
+            return
+
+        if event.keyval in (keysyms.KP_Enter,
+                            keysyms.Return):
             self._add_text_to_combo_list()
         
     def _on_child_entry__expose_event(self, widget, event):
