@@ -27,19 +27,43 @@
 
 import time
 
-import gtk
 import gobject
+import gtk
+from gtk import gdk
 
 from kiwi import ValueUnset
 from kiwi.datatypes import ValidationError, converter
 from kiwi.interfaces import Mixin, MixinSupportValidation
-from kiwi.ui.gadgets import set_background, merge_colors
+from kiwi.ui.gadgets import set_background
 
 MERGE_COLORS_DELAY = 100
 CURSOR_POS_CHECKING_DELAY = 200
 
 __pychecker__ = 'no-classattr'
 
+def merge_colors(widget, src_color, dst_color, steps=10):
+    """
+    Change the background of widget from src_color to dst_color
+    in the number of steps specified
+    """
+    gdk_src = gdk.color_parse(src_color)
+    gdk_dst = gdk.color_parse(dst_color)
+    rs, gs, bs = gdk_src.red, gdk_src.green, gdk_src.blue
+    rd, gd, bd = gdk_dst.red, gdk_dst.green, gdk_dst.blue
+    rinc = (rd - rs) / float(steps)
+    ginc = (gd - gs) / float(steps)
+    binc = (bd - bs) / float(steps)
+    for dummy in xrange(steps):
+        rs += rinc
+        gs += ginc
+        bs += binc
+        set_background(widget, "#%02X%02X%02X" % (int(rs) >> 8,
+                                                  int(gs) >> 8,
+                                                  int(bs) >> 8))
+        yield True
+
+    yield False
+
 class WidgetMixin(Mixin):
     """This class is a mixin that provide a common interface for KiwiWidgets.
 
@@ -49,18 +73,10 @@ class WidgetMixin(Mixin):
 
     def __init__(self, data_type=str, model_attribute=None,
                  default_value=None):
-        # we need initial values so this variables always exist
-        self._default_value = None
-        self._data_type = None
-        self._model_attribute = None
+        self._default_value = default_value
+        self._data_type = data_type
+        self._model_attribute = model_attribute
         self._data_format = None
-        # this attibute will be set to the view that owns the widget
-        self.owner = None
-        
-        # now we setup the variables with our parameters
-        self.set_data_type(data_type)
-        self.set_model_attribute(model_attribute)
-        self.set_default_value(default_value)
 
     def set_data_format(self, format):
         self._data_format = format
@@ -92,22 +108,22 @@ class WidgetMixin(Mixin):
         
     def do_get_property(self, pspec):
         prop_name = pspec.name.replace("-", "_")
-        func = getattr(self, "get_%s" % prop_name, None)
+        func = getattr(self, "prop_get_%s" % prop_name, None)
         if not func:
             raise AttributeError("Invalid property name: %s" % pspec.name)
         return func()
 
     def do_set_property(self, pspec, value):
         prop_name = pspec.name.replace("-", "_")
-        func = getattr(self, "set_%s" % prop_name)
+        func = getattr(self, "prop_set_%s" % prop_name)
         if not func:
             raise AttributeError("Invalid property name: %s" % pspec.name)
         return func(value)
     
-    def get_data_type(self):
+    def prop_get_data_type(self):
         return self._data_type
 
-    def set_data_type(self, obj):
+    def prop_set_data_type(self, obj):
         """Set the data type for the widget
         
         data_type can be None, a type object or a string with the name of the
@@ -133,16 +149,16 @@ class WidgetMixin(Mixin):
 
         self._data_type = data_type
 
-    def get_model_attribute(self):
+    def prop_get_model_attribute(self):
         return self._model_attribute
 
-    def set_model_attribute(self, attribute):
+    def prop_set_model_attribute(self, attribute):
         self._model_attribute = attribute
 
-    def get_default_value(self):
+    def prop_get_default_value(self):
         return self._default_value
 
-    def set_default_value(self, value):
+    def prop_set_default_value(self, value):
         if isinstance(value, basestring):
             value = self.str2type(value)
             
@@ -196,7 +212,7 @@ class WidgetMixinSupportValidation(WidgetMixin, MixinSupportValidation):
     The validatation feature provides a way to check the data entered and to
     display information about what is wrong.
     """
-    
+
     def __init__(self, data_type=str, model_attribute=None,
                  default_value=None, widget=None):
         WidgetMixin.__init__(self, data_type, model_attribute, default_value)
@@ -212,8 +228,6 @@ class WidgetMixinSupportValidation(WidgetMixin, MixinSupportValidation):
         # this is the last time the user changed the widget
         self._last_change_time = None
 
-        self._validation_error_message = ""
-        
         # id that paints the background red
         self._background_timeout_id = -1
         # id for idle that checks the cursor position
@@ -229,8 +243,6 @@ class WidgetMixinSupportValidation(WidgetMixin, MixinSupportValidation):
         self._draw_mandatory_icon = False
         
         self._draw_info_icon = False
-        self._show_error_tooltip = False
-        self._error_tooltip_visible = False    
 
         # this attribute stores the info on where to draw icons and paint
         # the background
@@ -238,13 +250,11 @@ class WidgetMixinSupportValidation(WidgetMixin, MixinSupportValidation):
             widget = self
         self._widget_to_draw = widget
 
-        self.owner = None
-        
-    def get_mandatory(self):
+    def prop_get_mandatory(self):
         """Checks if the Kiwi Widget is set to mandatory"""
         return self._mandatory
     
-    def set_mandatory(self, mandatory):
+    def prop_set_mandatory(self, mandatory):
         """Sets the Kiwi Widget as mandatory, in other words, 
         the user needs to provide data to the widget 
         """
@@ -253,14 +263,16 @@ class WidgetMixinSupportValidation(WidgetMixin, MixinSupportValidation):
     def before_validate(self, data):
         pass
     
-    def validate_data(self, data):
+    def validate_data(self, data, force=False):
         """Checks if the data is valid.
         
         Validates data-type and custom validation.
         data - the data to validate
         returns the widget data-type
         """
-
+        
+        old_state = self.is_correct()
+        
         # check if we should draw the man a stringdatory icon
         # this need to be done before any data conversion because we
         # we don't want to end drawing two icons
@@ -301,22 +313,23 @@ class WidgetMixinSupportValidation(WidgetMixin, MixinSupportValidation):
             # Show the error icon
             self._validation_error(e)
             data = ValueUnset
-            
-        # Step 3: Inform the user code wether their widgets
-        #         are valid or not
-        self._notify_validity()
+
+        # Step 3, if validation changed, emit a signal
+        #         unless force is used, then we're always emitting
+        new_state = self.is_correct()
+        if old_state == new_state and force == False:
+            return data
+        
+        self.emit('validation-changed', new_state)
 
         return data
     
     def _validation_error(self, e):
         if self._invalid_data:
             self._blank_data = False
-            # check if the remaining widgets are ok
-            self._notify_validity()
-        
+            
         self._invalid_data = True
-        self._validation_error_message = str(e)
-        self._error_tooltip.set_error_text(self._validation_error_message)
+        self._error_tooltip.set_error_text(str(e))
         if self._complaint_checker_id == -1:
             self._complaint_checker_id = \
                 gobject.idle_add(self._check_for_complaints)
@@ -328,24 +341,24 @@ class WidgetMixinSupportValidation(WidgetMixin, MixinSupportValidation):
         """Check for existing complaints and when to start complaining is case
         the input is wrong
         """
-        if self._last_change_time is None:
+        if not self._last_change_time:
             # the user has not started to type
             return True
         
-        now = time.time()
-        elapsed_time = now - self._last_change_time
-        
+        elapsed_time = time.time() - self._last_change_time
         if elapsed_time < COMPLAIN_DELAY:
             return True
         
         if not self._invalid_data:
             return True
-        
+
+        return self._maybe_draw_icon()
+    
+    def _maybe_draw_icon(self):
         # if we are already complaining, don't complain again
         if self._background_timeout_id != -1:
             return True
         
-        self._show_error_tooltip = True
         self._draw_info_icon = True
         self.queue_draw()
         func = merge_colors(self._widget_to_draw, 
@@ -365,7 +378,7 @@ class WidgetMixinSupportValidation(WidgetMixin, MixinSupportValidation):
             gobject.source_remove(self._complaint_checker_id)
             # before removing the get_cursor_position idle we need to be sure
             # that the tooltip is not been displayed
-            self._error_tooltip.disappear()
+            self._error_tooltip.hide()
             gobject.source_remove(self._get_cursor_position_id)
             self._background_timeout_id = -1
             self._complaint_checker_id = -1
@@ -378,7 +391,7 @@ class WidgetMixinSupportValidation(WidgetMixin, MixinSupportValidation):
         on top of the information icon
         """
         if not self._info_icon_position:
-            self._error_tooltip.disappear()
+            self._error_tooltip.hide()
             return True
         
         icon_x, icon_x_range, icon_y, icon_y_range = self._info_icon_position
@@ -387,20 +400,13 @@ class WidgetMixinSupportValidation(WidgetMixin, MixinSupportValidation):
         pointer_x, pointer_y = toplevel.get_pointer()
         
         if pointer_x not in icon_x_range or pointer_y not in icon_y_range:
-            self._error_tooltip.disappear()
+            self._error_tooltip.hide()
             return True
         
-        if self._error_tooltip.visible():
+        if self._error_tooltip.flags() & gtk.VISIBLE:
             return True
             
-        gdk_window = toplevel.window
-        window_x, window_y = gdk_window.get_origin()
-        entry_x, entry_y, entry_width, entry_height = self.get_allocation()
-        tooltip_width, tooltip_height = self._error_tooltip.get_size()
-        x = window_x + entry_x + entry_width - tooltip_width/2
-        y = window_y + entry_y - entry_height
-        self._error_tooltip.display(x, y)
-        self._error_tooltip_visible = True 
+        self._error_tooltip.display(toplevel, self)
                 
         return True
 
@@ -417,8 +423,7 @@ class WidgetMixinSupportValidation(WidgetMixin, MixinSupportValidation):
             return
             
         (iconx, icony,
-         pixbuf,
-         pixw, pixh) = self._render_icon(icon, self._widget_to_draw)
+         pixw, pixh, pixbuf) = self._render_icon(icon, self._widget_to_draw)
         
         self._draw_pixbuf(window, iconx, icony, pixbuf, pixw, pixh)
         
@@ -429,27 +434,21 @@ class WidgetMixinSupportValidation(WidgetMixin, MixinSupportValidation):
                                         icony, icony_range)
 
     def _render_icon(self, icon, widget):
+        widget_x, widget_y, widget_w, widget_h = widget.get_allocation()
         pixbuf = self.render_icon(icon, gtk.ICON_SIZE_MENU)
         pixw = pixbuf.get_width()
         pixh = pixbuf.get_height()
-        widget_x, widget_y, widget_w, widget_h = widget.get_allocation()
         
-        iconx = widget_x + widget_w - pixw
-        icony = widget_y + widget_h - pixh
-        
-        return iconx, icony, pixbuf, pixw, pixh
+        return (widget_x + widget_w - pixw,
+                widget_y + widget_h - pixh,
+                pixw, pixh, pixbuf)
 
     def _draw_pixbuf(self, window, iconx, icony, pixbuf, pixw, pixh):
         area_window = window.get_children()[0]
         winw, winh = area_window.get_size()
             
         area_window.draw_pixbuf(None, pixbuf, 0, 0, 
-                                winw - pixw, (winh - pixh)/2, 
-                                pixw, pixh)
-
-    def _notify_validity(self):
-        if self.owner:
-            self.owner.notify_validity()
+                                winw - pixw, (winh - pixh)/2, pixw, pixh)
 
     def is_correct(self):
         if self._invalid_data:
@@ -473,28 +472,25 @@ class ErrorTooltip(gtk.Window):
     def __init__(self, widget):
         gtk.Window.__init__(self, gtk.WINDOW_POPUP)
         
-        self._widget = widget
-        
         eventbox = gtk.EventBox()
         set_background(eventbox, "#fcffcd")
 
         alignment = gtk.Alignment()
         alignment.set_border_width(4)
-        self.label = gtk.Label()
-        alignment.add(self.label)
+        self._label = gtk.Label()
+        alignment.add(self._label)
         eventbox.add(alignment)
         self.add(eventbox)
 
     def set_error_text(self, text):
-        self.label.set_text(text)
+        self._label.set_text(text)
     
-    def display(self, x, y):
+    def display(self, window, widget):
+        window_x, window_y = window.window.get_origin()
+        entry_x, entry_y, entry_width, entry_height = widget.get_allocation()
+        tooltip_width, tooltip_height = self.get_size()
+        x = window_x + entry_x + entry_width - tooltip_width/2
+        y = window_y + entry_y - entry_height
+        
         self.move(x, y)
         self.show_all()
-        
-    def visible(self):
-        return self.get_property("visible")
-    
-    def disappear(self):
-        self.hide()
-
